@@ -24,7 +24,8 @@ try:
     import logging
 except:
     logging = None
-    
+
+from psycopg2 import extensions as _ext
 from psycopg2.extensions import cursor as _cursor
 from psycopg2.extensions import connection as _connection
 from psycopg2.extensions import adapt as _A
@@ -142,7 +143,7 @@ class RealDictConnection(_connection):
         if name is None:
             return _connection.cursor(self, cursor_factory=RealDictCursor)
         else:
-	    return _connection.cursor(self, name, cursor_factory=RealDictCursor)
+            return _connection.cursor(self, name, cursor_factory=RealDictCursor)
 
 class RealDictCursor(DictCursorBase):
     """A cursor that uses a real dict as the base type for rows.
@@ -223,10 +224,10 @@ class LoggingConnection(_connection):
             
     def cursor(self, name=None):
         self._check()
-	if name is None:
+        if name is None:
             return _connection.cursor(self, cursor_factory=LoggingCursor)
         else:
-	    return _connection.cursor(self, name, cursor_factory=LoggingCursor)
+            return _connection.cursor(self, name, cursor_factory=LoggingCursor)
 
 class LoggingCursor(_cursor):
     """A cursor that logs queries using its connection logging facilities."""
@@ -265,10 +266,10 @@ class MinTimeLoggingConnection(LoggingConnection):
 
     def cursor(self, name=None):
         self._check()
-	if name is None:
+        if name is None:
             return _connection.cursor(self, cursor_factory=MinTimeLoggingCursor)
         else:
-	    return _connection.cursor(self, name, cursor_factory=MinTimeLoggingCursor)
+            return _connection.cursor(self, name, cursor_factory=MinTimeLoggingCursor)
     
 class MinTimeLoggingCursor(LoggingCursor):
     """The cursor sub-class companion to MinTimeLoggingConnection."""
@@ -281,3 +282,77 @@ class MinTimeLoggingCursor(LoggingCursor):
         self.timestamp = time.time()
         return LoggingCursor.execute(self, procname, vars)
 
+
+# a dbtype and adapter for Python UUID type
+
+try:
+    import uuid
+
+    class UUID_adapter(object):
+        """Adapt Python's uuid.UUID type to PostgreSQL's uuid."""
+        
+        def __init__(self, uuid):
+            self._uuid = uuid
+    
+        def prepare(self, conn):
+            pass
+        
+        def getquoted(self):
+            return "'"+str(self._uuid)+"'::uuid"
+            
+        __str__ = getquoted
+
+    def register_uuid(oid=None):
+        """Create the UUID type and an uuid.UUID adapter."""
+        if not oid: oid = 2950
+        _ext.UUID = _ext.new_type((oid, ), "UUID",
+                lambda data, cursor: data and uuid.UUID(data) or None)
+        _ext.register_type(_ext.UUID)
+        _ext.register_adapter(uuid.UUID, UUID_adapter)
+        return _ext.UUID
+
+except ImportError, e:
+    def register_uuid(oid=None):
+        """Create the UUID type and an uuid.UUID adapter.
+
+        This is a fake function that will always raise an error because the
+        import of the uuid module failed.
+        """
+        raise e
+
+
+# a type, dbtype and adapter for PostgreSQL inet type
+
+class Inet(object):
+    """Wrap a string to allow for correct SQL-quoting of inet values.
+
+    Note that this adapter does NOT check the passed value to make
+    sure it really is an inet-compatible address but DOES call adapt()
+    on it to make sure it is impossible to execute an SQL-injection
+    by passing an evil value to the initializer.
+    """
+    def __init__(self, addr):
+        self.addr
+    
+    def prepare(self, conn):
+        self._conn = conn
+    
+    def getquoted(self):
+        obj = adapt(self.addr)
+        if hasattr(obj, 'prepare'):
+            obj.prepare(self._conn)
+        return obj.getquoted()+"::inet"
+
+    def __str__(self):
+        return str(self.addr)
+        
+def register_inet(oid=None):
+    """Create the INET type and an Inet adapter."""
+    if not oid: oid = 869
+    _ext.INET = _ext.new_type((oid, ), "INET",
+            lambda data, cursor: data and Inet(data) or None)
+    _ext.register_type(_ext.INET)
+    return _ext.INET
+
+
+__all__ = [ k for k in locals().keys() if not k.startswith('_') ]
